@@ -20,6 +20,10 @@
  * Author: Shaun McCance <shaunm@gnome.org>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
@@ -216,7 +220,6 @@ enum {
     COMPLETION_COL_TITLE,
     COMPLETION_COL_DESC,
     COMPLETION_COL_ICON,
-    COMPLETION_COL_PIXBUF,
     COMPLETION_COL_PAGE,
     COMPLETION_COL_FLAGS
 };
@@ -408,7 +411,7 @@ location_entry_constructed (GObject *object)
     priv->history = gtk_list_store_new (8,
                                         G_TYPE_STRING,  /* title */
                                         G_TYPE_STRING,  /* desc */
-                                        G_TYPE_OBJECT,  /* icon */
+                                        G_TYPE_STRING,  /* icon */
                                         G_TYPE_STRING,  /* uri */
                                         G_TYPE_STRING,  /* doc */
                                         G_TYPE_STRING,  /* page */
@@ -420,19 +423,16 @@ location_entry_constructed (GObject *object)
                       object);
     g_object_set (object, "model", priv->history, NULL);
     if (priv->enable_search) {
-        GdkPixbuf *pixbuf = yelp_settings_get_icon_pixbuf (yelp_settings_get_default (),
-                                                           "system-search");
         gtk_list_store_append (priv->history, &iter);
         gtk_list_store_set (priv->history, &iter,
                             HISTORY_COL_FLAGS, LOCATION_ENTRY_IS_SEPARATOR,
                             -1);
         gtk_list_store_append (priv->history, &iter);
         gtk_list_store_set (priv->history, &iter,
-                            HISTORY_COL_ICON, pixbuf,
+                            HISTORY_COL_ICON, "edit-find-symbolic",
                             HISTORY_COL_TITLE, _("Search..."),
                             HISTORY_COL_FLAGS, LOCATION_ENTRY_IS_SEARCH,
                             -1);
-        g_object_unref (pixbuf);
     }
 
     /* Set up the history drop-down */
@@ -455,7 +455,7 @@ location_entry_constructed (GObject *object)
     g_object_set (priv->icon_cell, "yalign", 0.2, NULL);
     gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (object),
                                     priv->icon_cell,
-                                    "pixbuf",
+                                    "icon-name",
                                     HISTORY_COL_ICON,
                                     NULL);
     gtk_cell_layout_reorder (GTK_CELL_LAYOUT (object), priv->icon_cell, 0);
@@ -670,15 +670,10 @@ location_entry_start_search (YelpLocationEntry *entry,
     if (!priv->enable_search)
         return;
     if (clear && !priv->search_mode) {
-        GtkTreeIter iter;
-        gchar *page;
-        gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->history), &iter);
-        gtk_tree_model_get (GTK_TREE_MODEL (priv->history), &iter,
-                            HISTORY_COL_PAGE, &page,
-                            -1);
-        if (!(page && g_str_has_prefix (page, "search=")))
+        const gchar *icon = gtk_entry_get_icon_name (GTK_ENTRY (priv->text_entry),
+                                                     GTK_ENTRY_ICON_PRIMARY);
+        if (!g_str_equal (icon, "folder-saved-search"))
             gtk_entry_set_text (GTK_ENTRY (priv->text_entry), "");
-        g_free (page);
     }
     priv->search_mode = TRUE;
     location_entry_set_entry (entry, FALSE);
@@ -743,8 +738,8 @@ location_entry_set_completion (YelpLocationEntry *entry,
     gtk_cell_layout_reorder (GTK_CELL_LAYOUT (priv->completion), icon_cell, 0);
     gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (priv->completion),
                                     icon_cell,
-                                    "pixbuf",
-                                    COMPLETION_COL_PIXBUF,
+                                    "icon-name",
+                                    COMPLETION_COL_ICON,
                                     NULL);
     if (priv->bookmarks) {
         bookmark_cell = gtk_cell_renderer_pixbuf_new ();
@@ -765,21 +760,15 @@ location_entry_set_entry (YelpLocationEntry *entry, gboolean emit)
     GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (entry));
     GtkTreePath *path = NULL;
     GtkTreeIter iter;
-    GdkPixbuf *pixbuf;
+    gchar *icon_name;
 
     if (priv->search_mode) {
-        YelpSettings *settings = yelp_settings_get_default ();
-        GdkPixbuf *pixbuf;
-        pixbuf = yelp_settings_get_icon_pixbuf (settings, "system-search");
-        gtk_entry_set_icon_from_pixbuf (GTK_ENTRY (priv->text_entry),
-                                        GTK_ENTRY_ICON_PRIMARY,
-                                        pixbuf);
-        g_object_unref (pixbuf);
-        pixbuf = yelp_settings_get_icon_pixbuf (settings, "edit-clear");
-        gtk_entry_set_icon_from_pixbuf (GTK_ENTRY (priv->text_entry),
-                                        GTK_ENTRY_ICON_SECONDARY,
-                                        pixbuf);
-        g_object_unref (pixbuf);
+        gtk_entry_set_icon_from_icon_name (GTK_ENTRY (priv->text_entry),
+                                           GTK_ENTRY_ICON_PRIMARY,
+                                           "edit-find-symbolic");
+        gtk_entry_set_icon_from_icon_name (GTK_ENTRY (priv->text_entry),
+                                           GTK_ENTRY_ICON_SECONDARY,
+                                           "edit-clear-symbolic");
         gtk_entry_set_icon_tooltip_text (GTK_ENTRY (priv->text_entry),
                                          GTK_ENTRY_ICON_SECONDARY,
                                          _("Clear the search text"));
@@ -800,47 +789,38 @@ location_entry_set_entry (YelpLocationEntry *entry, gboolean emit)
         gtk_tree_model_get_iter (model, &iter, path);
         gtk_tree_model_get (model, &iter,
                             HISTORY_COL_TITLE, &text,
-                            HISTORY_COL_ICON, &pixbuf,
+                            HISTORY_COL_ICON, &icon_name,
                             HISTORY_COL_FLAGS, &flags,
                             HISTORY_COL_DOC, &doc_uri,
                             HISTORY_COL_PAGE, &page_id,
                             -1);
         if (flags & LOCATION_ENTRY_IS_LOADING) {
-            GdkPixbuf *loading = yelp_settings_get_icon_pixbuf (yelp_settings_get_default (),
-                                                                "image-loading");
-            gtk_entry_set_icon_from_pixbuf (GTK_ENTRY (priv->text_entry),
-                                            GTK_ENTRY_ICON_PRIMARY,
-                                            loading);
-            g_object_unref (loading);
+            gtk_entry_set_icon_from_icon_name (GTK_ENTRY (priv->text_entry),
+                                               GTK_ENTRY_ICON_PRIMARY,
+                                               "image-loading-symbolic");
             if (priv->pulse > 0)
                 g_source_remove (priv->pulse);
             priv->pulse = g_timeout_add (80, location_entry_pulse, entry);
         }
         else {
-            gtk_entry_set_icon_from_pixbuf (GTK_ENTRY (priv->text_entry),
-                                            GTK_ENTRY_ICON_PRIMARY,
-                                            pixbuf);
+            gtk_entry_set_icon_from_icon_name (GTK_ENTRY (priv->text_entry),
+                                               GTK_ENTRY_ICON_PRIMARY,
+                                               icon_name);
         }
-        g_object_unref (pixbuf);
         if (priv->bookmarks && doc_uri && page_id) {
+            GdkPixbuf *pixbuf;
             if (!yelp_bookmarks_is_bookmarked (priv->bookmarks, doc_uri, page_id)) {
-                pixbuf = yelp_settings_get_icon_pixbuf (yelp_settings_get_default (),
-                                                        "yelp-bookmark-add");
-                gtk_entry_set_icon_from_pixbuf (GTK_ENTRY (priv->text_entry),
-                                                GTK_ENTRY_ICON_SECONDARY,
-                                                pixbuf);
-                g_object_unref (pixbuf);
+                gtk_entry_set_icon_from_icon_name (GTK_ENTRY (priv->text_entry),
+                                                   GTK_ENTRY_ICON_SECONDARY,
+                                                   "yelp-bookmark-add-symbolic");
                 gtk_entry_set_icon_tooltip_text (GTK_ENTRY (priv->text_entry),
                                                  GTK_ENTRY_ICON_SECONDARY,
                                                  _("Bookmark this page"));
             }
             else {
-                pixbuf = yelp_settings_get_icon_pixbuf (yelp_settings_get_default (),
-                                                        "yelp-bookmark-remove");
-                gtk_entry_set_icon_from_pixbuf (GTK_ENTRY (priv->text_entry),
-                                                GTK_ENTRY_ICON_SECONDARY,
-                                                pixbuf);
-                g_object_unref (pixbuf);
+                gtk_entry_set_icon_from_icon_name (GTK_ENTRY (priv->text_entry),
+                                                   GTK_ENTRY_ICON_SECONDARY,
+                                                   "yelp-bookmark-remove-symbolic");
                 gtk_entry_set_icon_tooltip_text (GTK_ENTRY (priv->text_entry),
                                                  GTK_ENTRY_ICON_SECONDARY,
                                                  _("Remove bookmark"));
@@ -1086,13 +1066,13 @@ cell_set_bookmark_icon (GtkCellLayout     *layout,
     YelpLocationEntryPrivate *priv = GET_PRIV (entry);
 
     if (priv->bookmarks == NULL) {
-        g_object_set (cell, "pixbuf", NULL, NULL);
+        g_object_set (cell, "icon-name", NULL, NULL);
         return;
     }
 
     gtk_tree_model_get (model, iter, HISTORY_COL_FLAGS, &flags, -1);
     if (flags & (LOCATION_ENTRY_IS_SEPARATOR | LOCATION_ENTRY_IS_SEARCH)) {
-        g_object_set (cell, "pixbuf", NULL, NULL);
+        g_object_set (cell, "icon-name", NULL, NULL);
         return;
     }
     
@@ -1101,14 +1081,10 @@ cell_set_bookmark_icon (GtkCellLayout     *layout,
                         HISTORY_COL_PAGE, &page_id,
                         -1);
     if (doc_uri && page_id &&
-        yelp_bookmarks_is_bookmarked (priv->bookmarks, doc_uri, page_id)) {
-        GdkPixbuf *pixbuf = yelp_settings_get_icon_pixbuf (yelp_settings_get_default (),
-                                                           "yelp-bookmark-remove");
-        g_object_set (cell, "pixbuf", pixbuf, NULL);
-        g_object_unref (pixbuf);
-    }
+        yelp_bookmarks_is_bookmarked (priv->bookmarks, doc_uri, page_id))
+        g_object_set (cell, "icon-name", "yelp-bookmark-remove-symbolic", NULL);
     else
-        g_object_set (cell, "pixbuf", NULL, NULL);
+        g_object_set (cell, "icon-name", NULL, NULL);
 
     g_free (doc_uri);
     g_free (page_id);
@@ -1130,15 +1106,10 @@ cell_set_completion_bookmark_icon (GtkCellLayout     *layout,
                             -1);
 
         if (page_id && yelp_bookmarks_is_bookmarked (priv->bookmarks,
-                                                     priv->completion_uri, page_id)) {
-            GdkPixbuf *pixbuf = yelp_settings_get_icon_pixbuf (yelp_settings_get_default (),
-                                                               "yelp-bookmark-remove");
-            g_object_set (cell, "pixbuf", pixbuf, NULL);
-            g_object_unref (pixbuf);
-        }
-        else {
-            g_object_set (cell, "pixbuf", NULL, NULL);
-        }
+                                                     priv->completion_uri, page_id))
+            g_object_set (cell, "icon-name", "yelp-bookmark-remove-symbolic", NULL);
+        else
+            g_object_set (cell, "icon-name", NULL, NULL);
 
         g_free (page_id);
     }
@@ -1187,6 +1158,12 @@ entry_match_func (GtkEntryCompletion *completion,
     gchar **strs;
     GtkTreeModel *model = gtk_entry_completion_get_model (completion);
     YelpLocationEntryPrivate *priv = GET_PRIV (entry);
+    static GRegex *nonword = NULL;
+
+    if (nonword == NULL)
+        nonword = g_regex_new ("\\W", 0, 0, NULL);
+    if (nonword == NULL)
+        return FALSE;
 
     gtk_tree_model_get (model, iter,
                         HISTORY_COL_TITLE, &title,
@@ -1201,7 +1178,7 @@ entry_match_func (GtkEntryCompletion *completion,
         g_free (desc);
     }
 
-    strs = g_strsplit (key, " ", -1);
+    strs = g_regex_split (nonword, key, 0);
     ret = TRUE;
     for (stri = 0; strs[stri]; stri++) {
         if (!titlecase || !strstr (titlecase, strs[stri])) {
@@ -1320,7 +1297,6 @@ view_loaded (YelpView          *view,
                                                      G_TYPE_STRING,  /* title */
                                                      G_TYPE_STRING,  /* desc */
                                                      G_TYPE_STRING,  /* icon */
-                                                     G_TYPE_OBJECT,  /* pixbuf */
                                                      G_TYPE_STRING,  /* uri */
                                                      G_TYPE_INT      /* flags */
                                                      );
@@ -1334,20 +1310,16 @@ view_loaded (YelpView          *view,
                 for (i = 0; ids[i]; i++) {
                     GtkTreeIter iter;
                     gchar *title, *desc, *icon;
-                    GdkPixbuf *pixbuf;
                     gtk_list_store_insert (GTK_LIST_STORE (base), &iter, 0);
                     title = yelp_document_get_page_title (document, ids[i]);
                     desc = yelp_document_get_page_desc (document, ids[i]);
                     icon = yelp_document_get_page_icon (document, ids[i]);
-                    pixbuf = yelp_settings_get_icon_pixbuf (yelp_settings_get_default (), icon);
                     gtk_list_store_set (base, &iter,
                                         COMPLETION_COL_TITLE, title,
                                         COMPLETION_COL_DESC, desc,
                                         COMPLETION_COL_ICON, icon,
-                                        COMPLETION_COL_PIXBUF, pixbuf,
                                         COMPLETION_COL_PAGE, ids[i],
                                         -1);
-                    g_object_unref (pixbuf);
                     g_free (icon);
                     g_free (desc);
                     g_free (title);
@@ -1404,17 +1376,13 @@ view_uri_selected (YelpView          *view,
     else {
         gint num;
         GtkTreeIter last;
-        GdkPixbuf *pixbuf;
-        pixbuf = yelp_settings_get_icon_pixbuf (yelp_settings_get_default (),
-                                                "help-contents");
         gtk_list_store_prepend (priv->history, &iter);
         gtk_list_store_set (priv->history, &iter,
                             HISTORY_COL_TITLE, _("Loading"),
-                            HISTORY_COL_ICON, pixbuf,
+                            HISTORY_COL_ICON, "help-contents",
                             HISTORY_COL_URI, struri,
                             HISTORY_COL_FLAGS, LOCATION_ENTRY_IS_LOADING,
                             -1);
-        g_object_unref (pixbuf);
         /* Limit to 15 entries. There are two extra for the search entry and
          * the separator above it.
          */
@@ -1506,20 +1474,16 @@ view_page_icon (YelpView          *view,
 {
     GtkTreeIter first;
     gchar *icon;
-    GdkPixbuf *pixbuf;
     YelpLocationEntryPrivate *priv = GET_PRIV (entry);
 
     g_object_get (view, "page-icon", &icon, NULL);
     if (icon == NULL)
         return;
 
-    pixbuf = yelp_settings_get_icon_pixbuf (yelp_settings_get_default (), icon);
-
     if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->history), &first))
         gtk_list_store_set (priv->history, &first,
-                            HISTORY_COL_ICON, pixbuf,
+                            HISTORY_COL_ICON, icon,
                             -1);
-    g_object_unref (pixbuf);
     g_free (icon);
 }
 
@@ -1543,25 +1507,18 @@ bookmarks_changed (YelpBookmarks      *bookmarks,
                             HISTORY_COL_PAGE, &page_id,
                             -1);
         if (this_uri && g_str_equal (this_uri, doc_uri) && page_id) {
-            GdkPixbuf *pixbuf;
             if (!yelp_bookmarks_is_bookmarked (priv->bookmarks, doc_uri, page_id)) {
-                pixbuf = yelp_settings_get_icon_pixbuf (yelp_settings_get_default (),
-                                                        "yelp-bookmark-add");
-                gtk_entry_set_icon_from_pixbuf (GTK_ENTRY (priv->text_entry),
-                                                GTK_ENTRY_ICON_SECONDARY,
-                                                pixbuf);
-                g_object_unref (pixbuf);
+                gtk_entry_set_icon_from_icon_name (GTK_ENTRY (priv->text_entry),
+                                                   GTK_ENTRY_ICON_SECONDARY,
+                                                   "yelp-bookmark-add-symbolic");
                 gtk_entry_set_icon_tooltip_text (GTK_ENTRY (priv->text_entry),
                                                  GTK_ENTRY_ICON_SECONDARY,
                                                  _("Bookmark this page"));
             }
             else {
-                pixbuf = yelp_settings_get_icon_pixbuf (yelp_settings_get_default (),
-                                                        "yelp-bookmark-remove");
-                gtk_entry_set_icon_from_pixbuf (GTK_ENTRY (priv->text_entry),
-                                                GTK_ENTRY_ICON_SECONDARY,
-                                                pixbuf);
-                g_object_unref (pixbuf);
+                gtk_entry_set_icon_from_icon_name (GTK_ENTRY (priv->text_entry),
+                                                   GTK_ENTRY_ICON_SECONDARY,
+                                                   "yelp-bookmark-remove-symbolic");
                 gtk_entry_set_icon_tooltip_text (GTK_ENTRY (priv->text_entry),
                                                  GTK_ENTRY_ICON_SECONDARY,
                                                  _("Remove bookmark"));
