@@ -504,7 +504,7 @@ yelp_view_get_property (GObject    *object,
             if (priv->page_icon)
                 g_value_set_string (value, priv->page_icon);
             else
-                g_value_set_string (value, "yelp-page");
+                g_value_set_string (value, "yelp-page-symbolic");
             break;
         case PROP_STATE:
             g_value_set_enum (value, priv->state);
